unit MainForm_cpu;

{$mode objfpc}



interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  ExtCtrls,

  modul_cpu, modul_ram, BitOp;


const
  Farbe_Hintergrund = 0;
  Farbe_Lesen = 1;
  Farbe_Schreiben = 2;

  Farb_Code_HRS       = $D3E0C4; // Gruen
  Farb_Code_ARS       = $C8E4DF; // Gelb
  Farb_Code_SRS       = $D1B6B5; // Lila
  Farb_Code_HW        = $BFB099; // Grau

  Farb_Code_Lesen     = $E2E2E2; // Grau
  Farb_Code_Schreiben = $C0C0FF; // Rot


type


  TRR_ARS = record { Allgemein Register Satz }
    A, F, B, C, D, E, H, L, AF, BC, DE, HL, BC_Inhalt, DE_Inhalt, HL_Inhalt : Byte;
  end;

  TRR_SRS = record { Spezial Register Satz }
    I, R, IX, IY, SP, PC, IX_Inhalt, IY_Inhalt, SP_Inhalt : Byte;
  end;


  TReg_Red = record { Register }
    HRS { Haupt Register Satz }, ARS { Alternativer Register Satz } :  TRR_ARS;
    SRS : TRR_SRS; { Spezial Register Satz }
    IFF1, IFF2 : Byte;
    interrupt_mode : Byte;
  end;


  { TForm_CPU }

  TForm_CPU = class(TForm)
    Image_CPU: TImage;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }

    Reg_Red : TReg_Red;
    procedure Red_Reset; { Ruecksetzen aller Markierungs-Register }
    procedure Register_anzeigen; { zeigt die CPU-Register im Formular an }
  end; { TForm_CPU }

var
  Form_CPU: TForm_CPU;

implementation


uses
  MainForm_EMUZ80;




procedure TForm_CPU.FormCreate(Sender: TObject);
begin

  mod_cpu := Tmod_cpu.Create;

  mod_cpu.Reset;

  Red_Reset;

end; { procedure TForm_CPU.FormCreate }






procedure TForm_CPU.Red_Reset; { Ruecksetzen aller Markierungs-Register }
begin

  { HRS }
  Reg_Red.HRS.A:=Farbe_Hintergrund; Reg_Red.HRS.F:=Farbe_Hintergrund;
  Reg_Red.HRS.AF:=Farbe_Hintergrund;
  Reg_Red.HRS.B:=Farbe_Hintergrund; Reg_Red.HRS.C:=Farbe_Hintergrund;
  Reg_Red.HRS.BC:=Farbe_Hintergrund;
  Reg_Red.HRS.BC_Inhalt:=Farbe_Hintergrund;
  Reg_Red.HRS.D:=Farbe_Hintergrund; Reg_Red.HRS.E:=Farbe_Hintergrund;
  Reg_Red.HRS.DE:=Farbe_Hintergrund;
  Reg_Red.HRS.DE_Inhalt:=Farbe_Hintergrund;
  Reg_Red.HRS.H:=Farbe_Hintergrund; Reg_Red.HRS.L:=Farbe_Hintergrund;
  Reg_Red.HRS.HL:=Farbe_Hintergrund;
  Reg_Red.HRS.HL_Inhalt:=Farbe_Hintergrund;

  { ARS }
  Reg_Red.ARS.A:=Farbe_Hintergrund; Reg_Red.ARS.F:=Farbe_Hintergrund;
  Reg_Red.ARS.AF:=Farbe_Hintergrund;
  Reg_Red.ARS.B:=Farbe_Hintergrund; Reg_Red.ARS.C:=Farbe_Hintergrund;
  Reg_Red.ARS.BC:=Farbe_Hintergrund;
  Reg_Red.ARS.BC_Inhalt:=Farbe_Hintergrund;
  Reg_Red.ARS.D:=Farbe_Hintergrund; Reg_Red.ARS.E:=Farbe_Hintergrund;
  Reg_Red.ARS.DE:=Farbe_Hintergrund;
  Reg_Red.ARS.DE_Inhalt:=Farbe_Hintergrund;
  Reg_Red.ARS.H:=Farbe_Hintergrund; Reg_Red.ARS.L:=Farbe_Hintergrund;
  Reg_Red.ARS.HL:=Farbe_Hintergrund;
  Reg_Red.ARS.HL_Inhalt:=Farbe_Hintergrund;

  { SRS }
  Reg_Red.SRS.I:=Farbe_Hintergrund;
  Reg_Red.SRS.R:=Farbe_Hintergrund;
  Reg_Red.SRS.IX:=Farbe_Hintergrund;
  Reg_Red.SRS.IX_Inhalt:=Farbe_Hintergrund;
  Reg_Red.SRS.IY:=Farbe_Hintergrund;
  Reg_Red.SRS.IY_Inhalt:=Farbe_Hintergrund;
  Reg_Red.SRS.SP:=Farbe_Hintergrund;
  Reg_Red.SRS.SP_Inhalt:=Farbe_Hintergrund;
  Reg_Red.SRS.PC:=Farbe_Hintergrund;

  { HW }
  Reg_Red.IFF1:=Farbe_Hintergrund;
  Reg_Red.IFF2:=Farbe_Hintergrund;

  mod_cpu.Reg.SRS.IX_d:=mod_cpu.Reg.SRS.IX;
  mod_cpu.Reg.SRS.IY_d:=mod_cpu.Reg.SRS.IY;

end; { procedure TForm_CPU.Red_Reset }



procedure TForm_CPU.Register_anzeigen; { zeigt die CPU-Register im Formular an }


  procedure Farbe_HRS(Farbe : Byte);
  begin
    with Image_CPU.Picture.Bitmap.Canvas do begin
      case Farbe of
        Farbe_Hintergrund : Brush.Color:=Farb_Code_HRS;
        Farbe_Lesen       : Brush.Color:=Farb_Code_Lesen;
        Farbe_Schreiben   : Brush.Color:=Farb_Code_Schreiben;
      end; { case Farbe of }
    end; { with Image_CPU.Picture.Bitmap.Canvas do }
  end; { procedure Farbe_HRS }




  procedure Farbe_ARS(Farbe : Byte);
  begin
    with Image_CPU.Picture.Bitmap.Canvas do begin
      case Farbe of
        Farbe_Hintergrund : Brush.Color:=Farb_Code_ARS;
        Farbe_Lesen       : Brush.Color:=Farb_Code_Lesen;
        Farbe_Schreiben   : Brush.Color:=Farb_Code_Schreiben;
      end; { case Farbe of }
    end; { with Image_CPU.Picture.Bitmap.Canvas do }
  end; { procedure Farbe_ARS }





  procedure Farbe_SRS(Farbe : Byte);
  begin
    with Image_CPU.Picture.Bitmap.Canvas do begin
      case Farbe of
        Farbe_Hintergrund : Brush.Color:=Farb_Code_SRS;
        Farbe_Lesen       : Brush.Color:=Farb_Code_Lesen;
        Farbe_Schreiben   : Brush.Color:=Farb_Code_Schreiben;
      end; { case Farbe of }
    end; { with Image_CPU.Picture.Bitmap.Canvas do }
  end; { procedure Farbe_HRS }







  procedure Farbe_HW(Farbe : Byte);
  begin
    with Image_CPU.Picture.Bitmap.Canvas do begin
      case Farbe of
        Farbe_Hintergrund : Brush.Color:=Farb_Code_HW;
        Farbe_Lesen       : Brush.Color:=Farb_Code_Lesen;
        Farbe_Schreiben   : Brush.Color:=Farb_Code_Schreiben;
      end; { case Farbe of }
    end; { with Image_CPU.Picture.Bitmap.Canvas do }
  end; { procedure Farbe_HRS }


begin

  with Image_CPU.Picture.Bitmap.Canvas do begin


    Font.Color:=clBlack;
    Font.Name:='Courier New';
    Font.Size:=16;

    { HRS }

    Farbe_HRS(Reg_Red.HRS.A);
    TextOut(37,49,IntToHex(mod_cpu.Reg.HRS.A,2)); // Reg A

    Farbe_HRS(Reg_Red.HRS.F);
    TextOut(106,49,IntToHex(mod_cpu.Reg.HRS.F,2)); // Reg F

    { Flag (7) = Sign (S) }
    if GetBit(mod_cpu.Reg.HRS.F,7) then
      TextOut(150,49,'1')
    else
      TextOut(150,49,'0');

    { Flag (6) = Zero (Z) }
    if GetBit(mod_cpu.Reg.HRS.F,6) then
      TextOut(175,49,'1')
    else
      TextOut(175,49,'0');

    { Flag (5) = (Y) }
    if GetBit(mod_cpu.Reg.HRS.F,5) then
      TextOut(200,49,'1')
    else
      TextOut(200,49,'0');

    { Flag (4) = Half-Carry (H) }
    if GetBit(mod_cpu.Reg.HRS.F,4) then
      TextOut(225,49,'1')
    else
      TextOut(225,49,'0');

    { Flag (3) = (X) }
    if GetBit(mod_cpu.Reg.HRS.F,3) then
      TextOut(250,49,'1')
    else
      TextOut(250,49,'0');

    { Flag (2) = Parity-Overflow (P) }
    if GetBit(mod_cpu.Reg.HRS.F,2) then
      TextOut(275,49,'1')
    else
      TextOut(275,49,'0');

    { Flag (1) = Num (N) }
    if GetBit(mod_cpu.Reg.HRS.F,1) then
      TextOut(300,49,'1')
    else
      TextOut(300,49,'0');

    { Flag (0) = Carry (C) }
    if GetBit(mod_cpu.Reg.HRS.F,0) then
      TextOut(325,49,'1')
    else
      TextOut(325,49,'0');


    Farbe_HRS(Reg_Red.HRS.B);
    TextOut(37,87,IntToHex(mod_cpu.Reg.HRS.B,2)); // Reg B

    Farbe_HRS(Reg_Red.HRS.C);
    TextOut(106,87,IntToHex(mod_cpu.Reg.HRS.C,2)); // Reg C

    Farbe_HRS(Reg_Red.HRS.BC);
    TextOut(190,87,IntToHex((mod_cpu.Reg.HRS.B shl 8) + mod_cpu.Reg.HRS.C,4)); // Reg BC

    Farbe_HRS(Reg_Red.HRS.BC_Inhalt);
    TextOut(314,87,IntToHex(mod_ram.Read_Byte((mod_cpu.Reg.HRS.B shl 8) + mod_cpu.Reg.HRS.C),2)); // (BC)

    Farbe_HRS(Reg_Red.HRS.D);
    TextOut(37,124,IntToHex(mod_cpu.Reg.HRS.D,2)); // Reg D

    Farbe_HRS(Reg_Red.HRS.E);
    TextOut(106,124,IntToHex(mod_cpu.Reg.HRS.E,2)); // Reg E

    Farbe_HRS(Reg_Red.HRS.DE);
    TextOut(190,124,IntToHex((mod_cpu.Reg.HRS.D shl 8) + mod_cpu.Reg.HRS.E,4)); // Reg DE

    Farbe_HRS(Reg_Red.HRS.DE_Inhalt);
    TextOut(314,124,IntToHex(mod_ram.Read_Byte((mod_cpu.Reg.HRS.D shl 8) + mod_cpu.Reg.HRS.E),2)); // (DE)

    Farbe_HRS(Reg_Red.HRS.H);
    TextOut(37,162,IntToHex(mod_cpu.Reg.HRS.H,2)); // Reg H

    Farbe_HRS(Reg_Red.HRS.L);
    TextOut(106,162,IntToHex(mod_cpu.Reg.HRS.L,2)); // Reg L

    Farbe_HRS(Reg_Red.HRS.HL);
    TextOut(190,162,IntToHex((mod_cpu.Reg.HRS.H shl 8) + mod_cpu.Reg.HRS.L,4)); // Reg HL

    Farbe_HRS(Reg_Red.HRS.HL_Inhalt);
    TextOut(314,162,IntToHex(mod_ram.Read_Byte((mod_cpu.Reg.HRS.H shl 8) + mod_cpu.Reg.HRS.L),2)); // (HL)




    { ARS }

    Farbe_ARS(Reg_Red.ARS.A);
    TextOut(37,51+198,IntToHex(mod_cpu.Reg.ARS.A,2)); // Reg A

    Farbe_ARS(Reg_Red.ARS.F);
    TextOut(106,51+198,IntToHex(mod_cpu.Reg.ARS.F,2)); // Reg F

    { Flag (7) = Sign (S) }
    if GetBit(mod_cpu.Reg.ARS.F,7) then
      TextOut(150,51+198,'1')
    else
      TextOut(150,51+198,'0');

    { Flag (6) = Zero (Z) }
    if GetBit(mod_cpu.Reg.ARS.F,6) then
      TextOut(175,51+198,'1')
    else
      TextOut(175,51+198,'0');

    { Flag (5) = (Y) }
    if GetBit(mod_cpu.Reg.ARS.F,5) then
      TextOut(200,51+198,'1')
    else
      TextOut(200,51+198,'0');

    { Flag (4) = Half-Carry (H) }
    if GetBit(mod_cpu.Reg.ARS.F,4) then
      TextOut(225,51+198,'1')
    else
      TextOut(225,51+198,'0');

    { Flag (3) = (X) }
    if GetBit(mod_cpu.Reg.ARS.F,3) then
      TextOut(250,51+198,'1')
    else
      TextOut(250,51+198,'0');

    { Flag (2) = Parity-Overflow (P) }
    if GetBit(mod_cpu.Reg.ARS.F,2) then
      TextOut(275,51+198,'1')
    else
      TextOut(275,51+198,'0');

    { Flag (1) = Num (N) }
    if GetBit(mod_cpu.Reg.ARS.F,1) then
      TextOut(300,51+198,'1')
    else
      TextOut(300,51+198,'0');

    { Flag (0) = Carry (C) }
    if GetBit(mod_cpu.Reg.ARS.F,0) then
      TextOut(325,51+198,'1')
    else
      TextOut(325,51+198,'0');

    Farbe_ARS(Reg_Red.ARS.B);
    TextOut(37,89+198,IntToHex(mod_cpu.Reg.ARS.B,2)); // Reg B

    Farbe_ARS(Reg_Red.ARS.C);
    TextOut(106,89+198,IntToHex(mod_cpu.Reg.ARS.C,2)); // Reg C

    Farbe_ARS(Reg_Red.ARS.BC);
    TextOut(190,89+198,IntToHex((mod_cpu.Reg.ARS.B shl 8) + mod_cpu.Reg.ARS.C,4)); // Reg BC

    Farbe_ARS(Reg_Red.ARS.BC_Inhalt);
    TextOut(314,89+198,IntToHex(mod_ram.Read_Byte((mod_cpu.Reg.ARS.B shl 8) + mod_cpu.Reg.ARS.C),2)); // (BC)

    Farbe_ARS(Reg_Red.ARS.D);
    TextOut(37,126+198,IntToHex(mod_cpu.Reg.ARS.D,2)); // Reg D

    Farbe_ARS(Reg_Red.ARS.E);
    TextOut(106,126+198,IntToHex(mod_cpu.Reg.ARS.E,2)); // Reg E

    Farbe_ARS(Reg_Red.ARS.DE);
    TextOut(190,126+198,IntToHex((mod_cpu.Reg.ARS.D shl 8) + mod_cpu.Reg.ARS.E,4)); // Reg DE

    Farbe_ARS(Reg_Red.ARS.DE_Inhalt);
    TextOut(314,126+198,IntToHex(mod_ram.Read_Byte((mod_cpu.Reg.ARS.D shl 8) + mod_cpu.Reg.ARS.E),2)); // (DE)

    Farbe_ARS(Reg_Red.ARS.H);
    TextOut(37,163+198,IntToHex(mod_cpu.Reg.ARS.H,2)); // Reg H

    Farbe_ARS(Reg_Red.ARS.L);
    TextOut(106,163+198,IntToHex(mod_cpu.Reg.ARS.L,2)); // Reg L

    Farbe_ARS(Reg_Red.ARS.HL);
    TextOut(190,163+198,IntToHex((mod_cpu.Reg.ARS.H shl 8) + mod_cpu.Reg.ARS.L,4)); // Reg HL

    Farbe_ARS(Reg_Red.ARS.HL_Inhalt);
    TextOut(314,163+198,IntToHex(mod_ram.Read_Byte((mod_cpu.Reg.ARS.H shl 8) + mod_cpu.Reg.ARS.L),2)); // (HL)


    { SRS }

    Farbe_SRS(Reg_Red.SRS.I);
    TextOut(121,430,IntToHex(mod_cpu.Reg.SRS.I,2)); // Reg I

    Farbe_SRS(Reg_Red.SRS.R);
    TextOut(270,430,IntToHex(mod_cpu.Reg.SRS.R,2)); // Reg R

    Farbe_SRS(Reg_Red.SRS.IX);
    TextOut(121,464,IntToHex(mod_cpu.Reg.SRS.IX,4)); // Reg IX

    Farbe_SRS(Reg_Red.SRS.IX_Inhalt);
    TextOut(270,464,IntToHex(mod_ram.Read_Byte(mod_cpu.Reg.SRS.IX_d),2)); // (IX+d)

    Farbe_SRS(Reg_Red.SRS.IY);
    TextOut(121,497,IntToHex(mod_cpu.Reg.SRS.IY,4)); // Reg IY

    Farbe_SRS(Reg_Red.SRS.IY_Inhalt);
    TextOut(270,497,IntToHex(mod_ram.Read_Byte(mod_cpu.Reg.SRS.IY_d),2)); // (IY+d)

    Farbe_SRS(Reg_Red.SRS.SP);
    TextOut(121,533,IntToHex(mod_cpu.Reg.SRS.SP,4)); // Reg SP

    Farbe_SRS(Reg_Red.SRS.SP_Inhalt);
    TextOut(243,533,IntToHex(mod_ram.Read_Byte(mod_cpu.Reg.SRS.SP),2)); // (SP)

    Farbe_SRS(Reg_Red.SRS.PC);
    TextOut(121,567,IntToHex(mod_cpu.Reg.SRS.PC,4)); // Reg PC
    TextOut(243,567,IntToHex(mod_ram.Read_Byte(mod_cpu.Reg.SRS.PC),2)); // (PC)



    { HW }

    Font.Size:=8;

    Farbe_HW(Reg_Red.IFF1);
    if mod_cpu.Reg.IFF1 then
      TextOut(330,419,'1') // IFF1=true
    else
      TextOut(330,419,'0'); // IFF1=false


    Farbe_HW(Reg_Red.IFF2);
    if mod_cpu.Reg.IFF2 then
      TextOut(330,463,'1') // IFF2=true
    else
      TextOut(330,463,'0'); // IFF2=false

    TextOut(320,490,'INT-'); // INT-
    TextOut(320,503,'Mode'); // Mode

    Font.Size:=12;
    TextOut(328,517,IntToStr(mod_cpu.Reg.interrupt_mode)); // IM


  end; { with Image_CPU.Picture.Bitmap.Canvas do }


end; { procedure TForm_CPU.Register_anzeigen }





procedure TForm_CPU.FormDestroy(Sender: TObject);
begin

  Form_EMUZ80.CPU_Left:=Form_CPU.Left;
  Form_EMUZ80.CPU_Top:=Form_CPU.Top;

  mod_cpu.Free;

end; { procedure TForm_CPU.FormDestroy }






initialization
  {$I mainform_cpu.lrs}

end.

